
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#include <stdlib.h>
#include <assert.h>

#include "ablock.h"
#include "xmalloc.h"

struct ablock_t *main_ablock = NULL;


struct ablock_t *ablock_new(struct nfa_t *nfa)
{
	struct ablock_t *ablock;

	ablock = xmalloc(sizeof(struct ablock_t));

	assert(nfa != NULL);
	ablock->nfa = nfa;

	ablock->head = NULL;
	ablock->tail = NULL;

	set_new( &(ablock->st_pres), nfa->nb_states);
	set_new( &(ablock->st_next), nfa->nb_states);

	return ablock;
}


void ablock_del(struct ablock_t *ablock)
{
	if (ablock != NULL)
	{
		while (ablock->head != NULL)
		{
			ablock_conj_del( ablock, ablock->head );
		}
		assert(ablock->tail == NULL);

		set_free( &(ablock->st_pres) );
		set_free( &(ablock->st_next) );

		free( ablock );
	}
	return;
}


void ablock_conj_new(struct ablock_t *ablock)
{
	struct ablock_conj_t *p;

	p = xmalloc(sizeof(struct ablock_conj_t));
	p->conj_id = 0;
	p->child1 = NULL;
	p->child2 = NULL;

	assert(ablock != NULL);
	p->prev = ablock->tail;
	p->next = NULL;

	if (ablock->tail != NULL)
	{
		ablock->tail->next = p;
	}
	else
	{
		assert(ablock->head == NULL);
		ablock->head = p;
	}
	ablock->tail = p;

	return;
}


void ablock_conj_del(struct ablock_t *ablock, struct ablock_conj_t *p)
{
	struct ablock_conj_t *prev, *next;

	assert(ablock != NULL);
	assert(p != NULL);

	assert(p->child1 != NULL);
	assert(p->child2 != NULL);
	ablock_del(p->child1);
	ablock_del(p->child2);

	prev = p->prev;
	next = p->next;

	if (prev != NULL)
	{
		prev->next = next;
	}
	else
	{
		assert(ablock->head == p);
		ablock->head = next;
	}
	if (next != NULL)
	{
		next->prev = prev;
	}
	else
	{
		assert(ablock->tail == p);
		ablock->tail = prev;
	}

	free(p);
	return;
}
