
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#ifndef _ABLOCK_H
#define _ABLOCK_H

/* activation blocks mgmt */

#include "nfa.h"
#include "set.h"


struct ablock_t
{
	struct nfa_t *nfa;						/* nfa of this activation */
	struct set_t st_pres, st_next;					/* present states and next states */
	struct ablock_conj_t *head, *tail;				/* subordinated activations */
};

/* several parallel stacks as doubly-linked list of binary trees */
struct ablock_conj_t
{
	unsigned long conj_id;						/* state of the master conjonction */
	struct ablock_t *child1, *child2;				/* child conjunctions */
	struct ablock_conj_t *prev, *next;				/* doubly-linked list */
};

extern struct ablock_t *main_ablock;					/* main activation block */


struct ablock_t *ablock_new(struct nfa_t *nfa);				/* create new ablock and initialize with nfa */
void ablock_del(struct ablock_t *ablock);				/* recursively delete */

void ablock_conj_new(struct ablock_t *ablock);				/* create a conj and append at end of list */
void ablock_conj_del(struct ablock_t *ablock, struct ablock_conj_t *p);	/* delete p and remove from ablock's list */

#endif
