
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#ifndef _ATOM_H
#define _ATOM_H

/* atomic line pattern matching, with no operators */

#include <sys/types.h>
#include <regex.h>


struct flags_t			/* how an atomic regex is matched */
{
	enum
	{
		flag_f,		/* fixed string, as in "fgrep" */
		flag_r,		/* regular regex, as in "grep" */
		flag_e		/* extended regex, as in "egrep" */
	} type;
	enum
	{
		flag_c,		/* case sensitive match */
		flag_i		/* case-insensitive match, as in "grep -i" */
	} kase;
	enum
	{
		flag_s,		/* find a substring of the input line */
		flag_x		/* whole line match, as in "grep -x" */
	} line;
};

#define DEFAULT_FLAG_TYPE	flag_r
#define DEFAULT_FLAG_KASE	flag_c
#define DEFAULT_FLAG_LINE	flag_s


struct atom_t
{
	unsigned int ref_cnt;
	struct flags_t flags;
	unsigned long lineno;	/* within the template file */
	char *str;		/* verbatim from the template file */
	regex_t *preg;		/* (preg == NULL) iff (flags.type == flag_f) */
};


struct atom_t *atom_new(struct flags_t flags, unsigned long lineno, char *str);

int atom_match(char *line, size_t lg, struct atom_t *ap);

void atom_free(struct atom_t **app);

#endif
