
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include "expression.h"

/*
	ref <stdarg.h> and the "C FAQ"
	There is no portable way of constructing a va_list structure at run-time
	Therefore, I cannot use vprintf :(

	This works only if sizeof(char *) == sizeof(int)
	so the compiler always generates "pushes" of the same size.
*/


#define pval(v)		(((v).vtype == t_str) ? (v).u.vstr : ((char *) ((v).u.vint)))


int expr_sprintf(char *buf, unsigned long bufsiz, struct expression_t *e, unsigned long nargs)
{
	int r;

	assert(buf != NULL);
	assert(bufsiz != 0);
	assert(e != NULL);
	assert(e->opc >= 1 && e->opc <= FCT_MAXARGS);
	assert(nargs >= 1 && nargs <= e->opc);
	assert(e->vop != NULL);
	assert(e->vop[0].vtype == t_str);

	switch ((unsigned char) nargs)
	{
		case  1:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr);
			break;
		case  2:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]));
			break;
		case  3:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]));
			break;
		case  4:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]));
			break;
		case  5:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]));
			break;
		case  6:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]));
			break;
		case  7:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]));
			break;
		case  8:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]));
			break;
		case  9:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]));
			break;
		case 10:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]));
			break;
		case 11:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]));
			break;
		case 12:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]));
			break;
		case 13:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]));
			break;
		case 14:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]));
			break;
		case 15:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]));
			break;
		case 16:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]));
			break;
		case 17:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]));
			break;
		case 18:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]));
			break;
		case 19:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]));
			break;
		case 20:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]));
			break;
		case 21:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]));
			break;
		case 22:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]));
			break;
		case 23:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]));
			break;
		case 24:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]));
			break;
		case 25:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]));
			break;
		case 26:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]));
			break;
		case 27:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]));
			break;
		case 28:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]));
			break;
		case 29:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]));
			break;
		case 30:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]));
			break;
		case 31:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]));
			break;
		case 32:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]));
			break;
		case 33:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]));
			break;
		case 34:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]));
			break;
		case 35:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]));
			break;
		case 36:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]));
			break;
		case 37:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]));
			break;
		case 38:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]));
			break;
		case 39:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]));
			break;
		case 40:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]));
			break;
		case 41:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]));
			break;
		case 42:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]));
			break;
		case 43:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]));
			break;
		case 44:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]));
			break;
		case 45:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]));
			break;
		case 46:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]));
			break;
		case 47:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]));
			break;
		case 48:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]));
			break;
		case 49:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]));
			break;
		case 50:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]),
			             pval(e->vop[49]));
			break;
		case 51:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]),
			             pval(e->vop[49]),
			             pval(e->vop[50]));
			break;
		case 52:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]),
			             pval(e->vop[49]),
			             pval(e->vop[50]),
			             pval(e->vop[51]));
			break;
		case 53:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]),
			             pval(e->vop[49]),
			             pval(e->vop[50]),
			             pval(e->vop[51]),
			             pval(e->vop[52]));
			break;
		case 54:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]),
			             pval(e->vop[49]),
			             pval(e->vop[50]),
			             pval(e->vop[51]),
			             pval(e->vop[52]),
			             pval(e->vop[53]));
			break;
		case 55:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]),
			             pval(e->vop[49]),
			             pval(e->vop[50]),
			             pval(e->vop[51]),
			             pval(e->vop[52]),
			             pval(e->vop[53]),
			             pval(e->vop[54]));
			break;
		case 56:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]),
			             pval(e->vop[49]),
			             pval(e->vop[50]),
			             pval(e->vop[51]),
			             pval(e->vop[52]),
			             pval(e->vop[53]),
			             pval(e->vop[54]),
			             pval(e->vop[55]));
			break;
		case 57:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]),
			             pval(e->vop[49]),
			             pval(e->vop[50]),
			             pval(e->vop[51]),
			             pval(e->vop[52]),
			             pval(e->vop[53]),
			             pval(e->vop[54]),
			             pval(e->vop[55]),
			             pval(e->vop[56]));
			break;
		case 58:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]),
			             pval(e->vop[49]),
			             pval(e->vop[50]),
			             pval(e->vop[51]),
			             pval(e->vop[52]),
			             pval(e->vop[53]),
			             pval(e->vop[54]),
			             pval(e->vop[55]),
			             pval(e->vop[56]),
			             pval(e->vop[57]));
			break;
		case 59:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]),
			             pval(e->vop[49]),
			             pval(e->vop[50]),
			             pval(e->vop[51]),
			             pval(e->vop[52]),
			             pval(e->vop[53]),
			             pval(e->vop[54]),
			             pval(e->vop[55]),
			             pval(e->vop[56]),
			             pval(e->vop[57]),
			             pval(e->vop[58]));
			break;
		case 60:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]),
			             pval(e->vop[49]),
			             pval(e->vop[50]),
			             pval(e->vop[51]),
			             pval(e->vop[52]),
			             pval(e->vop[53]),
			             pval(e->vop[54]),
			             pval(e->vop[55]),
			             pval(e->vop[56]),
			             pval(e->vop[57]),
			             pval(e->vop[58]),
			             pval(e->vop[59]));
			break;
		case 61:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]),
			             pval(e->vop[49]),
			             pval(e->vop[50]),
			             pval(e->vop[51]),
			             pval(e->vop[52]),
			             pval(e->vop[53]),
			             pval(e->vop[54]),
			             pval(e->vop[55]),
			             pval(e->vop[56]),
			             pval(e->vop[57]),
			             pval(e->vop[58]),
			             pval(e->vop[59]),
			             pval(e->vop[60]));
			break;
		case 62:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]),
			             pval(e->vop[49]),
			             pval(e->vop[50]),
			             pval(e->vop[51]),
			             pval(e->vop[52]),
			             pval(e->vop[53]),
			             pval(e->vop[54]),
			             pval(e->vop[55]),
			             pval(e->vop[56]),
			             pval(e->vop[57]),
			             pval(e->vop[58]),
			             pval(e->vop[59]),
			             pval(e->vop[60]),
			             pval(e->vop[61]));
			break;
		case 63:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]),
			             pval(e->vop[49]),
			             pval(e->vop[50]),
			             pval(e->vop[51]),
			             pval(e->vop[52]),
			             pval(e->vop[53]),
			             pval(e->vop[54]),
			             pval(e->vop[55]),
			             pval(e->vop[56]),
			             pval(e->vop[57]),
			             pval(e->vop[58]),
			             pval(e->vop[59]),
			             pval(e->vop[60]),
			             pval(e->vop[61]),
			             pval(e->vop[62]));
			break;
		case 64:
			r = snprintf(buf, bufsiz, e->vop[0].u.vstr,
			             pval(e->vop[ 1]),
			             pval(e->vop[ 2]),
			             pval(e->vop[ 3]),
			             pval(e->vop[ 4]),
			             pval(e->vop[ 5]),
			             pval(e->vop[ 6]),
			             pval(e->vop[ 7]),
			             pval(e->vop[ 8]),
			             pval(e->vop[ 9]),
			             pval(e->vop[10]),
			             pval(e->vop[11]),
			             pval(e->vop[12]),
			             pval(e->vop[13]),
			             pval(e->vop[14]),
			             pval(e->vop[15]),
			             pval(e->vop[16]),
			             pval(e->vop[17]),
			             pval(e->vop[18]),
			             pval(e->vop[19]),
			             pval(e->vop[20]),
			             pval(e->vop[21]),
			             pval(e->vop[22]),
			             pval(e->vop[23]),
			             pval(e->vop[24]),
			             pval(e->vop[25]),
			             pval(e->vop[26]),
			             pval(e->vop[27]),
			             pval(e->vop[28]),
			             pval(e->vop[29]),
			             pval(e->vop[30]),
			             pval(e->vop[31]),
			             pval(e->vop[32]),
			             pval(e->vop[33]),
			             pval(e->vop[34]),
			             pval(e->vop[35]),
			             pval(e->vop[36]),
			             pval(e->vop[37]),
			             pval(e->vop[38]),
			             pval(e->vop[39]),
			             pval(e->vop[40]),
			             pval(e->vop[41]),
			             pval(e->vop[42]),
			             pval(e->vop[43]),
			             pval(e->vop[44]),
			             pval(e->vop[45]),
			             pval(e->vop[46]),
			             pval(e->vop[47]),
			             pval(e->vop[48]),
			             pval(e->vop[49]),
			             pval(e->vop[50]),
			             pval(e->vop[51]),
			             pval(e->vop[52]),
			             pval(e->vop[53]),
			             pval(e->vop[54]),
			             pval(e->vop[55]),
			             pval(e->vop[56]),
			             pval(e->vop[57]),
			             pval(e->vop[58]),
			             pval(e->vop[59]),
			             pval(e->vop[60]),
			             pval(e->vop[61]),
			             pval(e->vop[62]),
			             pval(e->vop[63]));
			break;

		default:
			/* should never reach this point */
			assert(0);
			r = 0;
			break;
	}

	return r;
}
