
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#ifndef _EXPRESSION_H
#define _EXPRESSION_H

#include "idtable.h"
#include "evalue.h"


struct expression_t
{
	enum expr_node_t
	{
		n_comma,		/* , */
		n_cond,			/* ?: */
		n_logor,		/* || */
		n_logand,		/* && */
		n_bitor,		/* | */
		n_bitand,		/* & */
		n_not,			/* ! */
		n_compl,		/* ~ */
		n_plus,			/* + unary */
		n_minus,		/* - unary */
		n_xor,			/* ^ */
		n_eq,			/* == */
		n_ne,			/* != */
		n_le,			/* <= */
		n_ge,			/* >= */
		n_lt,			/* < */
		n_gt,			/* > */
		n_lshift,		/* << */
		n_rshift,		/* >> */
		n_add,			/* + binary */
		n_sub,			/* - binary */
		n_mul,			/* * */
		n_div,			/* / */
		n_mod,			/* % */
		n_incrs,		/* ++ suffix */
		n_decrs,		/* -- suffix */
		n_incrp,		/* ++ prefix */
		n_decrp,		/* -- prefix */
		n_index,		/* [] */
		n_assignment,		/* = */
		n_ainc,			/* += */
		n_adec,			/* -= */
		n_amul,			/* *= */
		n_adiv,			/* /= */
		n_amod,			/* %= */
		n_axor,			/* ^= */
		n_aor,			/* |= */
		n_aand,			/* &= */
		n_lshft,		/* <<= */
		n_rshft,		/* >>= */
		n_id,
		n_intcst,
		n_strcst
	} expr_node;

	enum vtype_t vtype;

	union
	{
		struct idtable_t *id;	/* n_id */
		long int vint;		/* n_intcst */
		char *vstr;		/* n_strcst */
	} u;

	/* operands vector */
	unsigned int opc;
	struct expression_t **opv;
	struct evalue_t *vop;
};

struct expression_t *expr_new(enum expr_node_t expr_node);
void expr_init(struct expression_t *e, unsigned int opc);
void expr_free(struct expression_t *root);

char *expr_eval_str(struct expression_t *e);
long int expr_eval_int(struct expression_t *e);
void expr_eval(struct expression_t *e, struct evalue_t *evalue);
void expr_eval_fct(struct expression_t *e, struct evalue_t *v);

char *image_int(long int n);
/* returns static memory */

int expr_sprintf(char *buf, unsigned long bufsiz, struct expression_t *e, unsigned long nargs);

/* max nb of arguments for functions */
#define FCT_MAXARGS	64
/* mandatory for "printf" functions: compiler always generates "pushes" of the same size. */
#define FCT_SIZEARG	(sizeof(char *) == sizeof(long int))

#endif
