
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#ifndef _IDTABLE_H
#define _IDTABLE_H

#include "avl.h"
#include "evalue.h"
#include "pattern.h"
#include "statement.h"


struct idtable_t
{
	enum id_kind_t
	{
		id_macro,
		id_var,					/* scalar variable */
		id_fct,
		id_tab					/* associative array */
	} id_kind;

	union
	{
		struct					/* id_macro */
		{
			struct pattern_t *v_pat;
			struct statement_t *v_stat;
		} s1;

		struct evalue_t v_var;			/* id_var */

		struct					/* id_fct */
		{
			enum vtype_t vtype;

			enum pred_fct_t
			{
				f_fflush,
				f_field,
				f_filename,
				f_fprintf,
				f_index,
				f_inttostr,
				f_length,
				f_line,
				f_lineno,
				f_match,
				f_nf,
				f_printf,
				f_split,
				f_sprintf,
				f_stderr,
				f_stdout,
				f_strcat,
				f_strtoint,
				f_strtonum,
				f_substr,
				f_system,
				f_tolower,
				f_toupper
			} v_fct;
		} s2;

		struct
		{
			enum vtype_t vtype;
			struct avl_t *root;
		} v_tab;
	} u;
};


struct idtable_t *idtable_new(enum id_kind_t id_kind);

void idtable_init(struct avl_t **root);			/* create entries for predefined IDs */
extern struct idtable_t *pred_fs;			/* predefined var FS */

void idtable_free(struct idtable_t *root);

#endif
