
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#ifndef _LEXIC_H
#define _LEXIC_H

#include <stdio.h>

extern char *template_fn;
extern FILE *template_fp;

enum lexic_mode_t
{
	lexic_pattern,		/* character ':' returns TOK_REGEX in pattern */
	lexic_expression	/* character ':' returns token ';' in conditional expression */
};
extern enum lexic_mode_t lexic_mode;

union lval_t
{
	char *string;
	unsigned long int number;
};

void get_token(void);
void get_flag(void);		/* returns letters as token (versus TOK_ID) */

extern int token;		/* may return EOF, unsigned char, or one of the TOKENs below */
extern union lval_t lval;

#define TOK_STRCST	300	/* string characters returned in lval.string */
#define TOK_INTCST	301	/* int value returned in lval.number */
#define TOK_REGEX	302	/* regex string returned in lval.string */
#define TOK_ID		303	/* ID string returned in lval.string */

/* reserved words wrt template */
#define TOK_BEGIN	400	/* BEGIN */
#define TOK_DECL	401	/* DECL */
#define TOK_FAILURE	402	/* FAILURE */
#define TOK_INT		403	/* int */
#define TOK_MACRO	404	/* macro */
#define TOK_STR		405	/* str */
#define TOK_SUCCESS	406	/* SUCCESS */

/* reserved words wrt statements */
#define TOK_ACCEPT	450
#define TOK_BREAK	451
#define TOK_CONTINUE	452
#define TOK_DELETE	453
#define TOK_DO		454
#define TOK_ELSE	455
#define TOK_EXIT	456
#define TOK_FOR		457
#define TOK_FORALL	458
#define TOK_IF		459
#define TOK_REJECT	460
#define TOK_WHILE	461

/* operators */
#define TOK_INCR	500	/* ++ */
#define TOK_DECR	501	/* -- */
#define TOK_LE		502	/* <= */
#define TOK_GE		503	/* >= */
#define TOK_EQ		504	/* == */
#define TOK_NE		505	/* != */
#define TOK_AND		506	/* && */
#define TOK_OR		507	/* || */
#define TOK_AINCR	508	/* += */
#define TOK_ADECR	509	/* -= */
#define TOK_AMULT	510	/* *= */
#define TOK_ADIV	511	/* /= */
#define TOK_AMOD	512	/* %= */
#define TOK_AXOR	513	/* ^= */
#define TOK_ALSH	514	/* <<= */
#define TOK_ARSH	515	/* >>= */
#define TOK_AAND	516	/* &= */
#define TOK_AOR		517	/* |= */
#define TOK_LSHIFT	519	/* << */
#define TOK_RSHIFT	520	/* >> */


long int str2int(char *str);

void abend(char *msg);

#endif
