
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

/*
	$ /usr/games/fortune -m comment
	%
	Real computer scientists don't comment their code.  The identifiers are
	so long they can't afford the disk space.
	%
	Real programmers don't comment their code.  It was hard to write, it
	should be hard to understand.
	%
	"Why waste negative entropy on comments, when you could use the same
	entropy to create bugs instead?"
	-- Steve Elias
	%
	"If the code and the comments disagree, then both are probably wrong."
	-- Norm Schryer
	%
	comment:
		A superfluous element of a source program included so the
		programmer can remember what the hell it was he was doing
		six months later.  Only the weak-minded need them, according
		to those who think they aren't.
*/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/param.h>
#include <assert.h>

#include "options.h"
#include "lexic.h"
#include "io.h"
#include "parse.h"
#include "expression.h"
#include "nfa.h"
#include "dump.h"
#include "hawk.h"
#include "xmalloc.h"

char version[] = "hawk (Herv's awk) v1.0, June 2008.";
char copyright[] = "Copyright (C) 2008 by Denis Valois && Cdric Llorens under the CeCILL license.";


int main(int argc, char *argv[])
{
	FILE *input_fp;
	int status = 1;
	char this_path[MAXPATHLEN + 1], *cp;
	unsigned long i;
	unsigned long mem_start = 0, mem_stop;

	assert(FCT_SIZEARG);
	mem_start = xinuse();
	options(argc, argv);

	if (opt_egg)
	{
		fprintf(stdout, "%s\n%s\n", version, copyright);
		fflush(stdout);
	}

	assert(template_fn != NULL);
	if ((input_fp = io_open(template_fn)) != NULL) template_fp = input_fp;
	parse_template();
	io_close(template_fp, io_filename);

	opt_v_free();

	gen_nfa();

	if (opt_debug)
	{
		if (decl != NULL)
		{
			fprintf(DUMP_FP, "\nDECL\n");
			dump_avl( decl );
		}

		if (begin != NULL)
		{
			fprintf(DUMP_FP, "\nBEGIN\n");
			dump_statement( begin );
		}

		fprintf(DUMP_FP, "\nPATTERN\n");
		dump_pattern( pattern );

		fprintf(DUMP_FP, "\nnb NFA\t%lu\n", nb_nfas);
		fprintf(DUMP_FP, "\tmain\t%p\n", (void *) main_nfa);
		for (i = 0; i < nb_nfas; i++)
		{
			fprintf(DUMP_FP, "NFA\t%lu\t%p\n", i, (void *) nfas[i]);
			dump_nfa( nfas[i] );
		}

		if (success != NULL)
		{
			fprintf(DUMP_FP, "\nSUCCESS\n");
			dump_statement( success );
		}

		if (failure != NULL)
		{
			fprintf(DUMP_FP, "\nFAILURE\n");
			dump_statement( failure );
		}
	}

	assert(input_fn != NULL);
	for (/* noop */; *input_fn != NULL; input_fn++)
	{
		if (opt_indirect_input)
		{
			input_fp = io_open(*input_fn);

			while (fgets(this_path, sizeof(this_path), input_fp) != NULL)
			{
				if ((cp = strchr(this_path, '\n')) != NULL) *cp = '\0';
				status &= hawk(this_path);
			}

			io_close(input_fp, *input_fn);
		}
		else
		{
			status &= hawk(*input_fn);
		}
	}

	if (opt_memory)
	{
		if ((mem_stop = xinuse()) != 0)
		{
			assert(mem_stop > mem_start);
			fprintf(DUMP_FP, "\nSTORAGE USE = %lu bytes\n", mem_stop - mem_start);
		}

		for (i = 0; i < nb_nfas; i++) nfa_free( nfas[i] );
		free( nfas );

		pattern_free( pattern );

		statement_free( &begin );
		statement_free( &success );
		statement_free( &failure );

		avl_free( decl );

		assert(template_fn != NULL);
		assert(io_line != NULL);
		free( template_fn );
		free( io_line );

		if (mem_stop != 0)
		{
			mem_stop = xinuse();
			assert(mem_stop >= mem_start);
			fprintf(DUMP_FP, "MEMORY LEAK = %lu bytes\n", mem_stop - mem_start);
		}
	}

	exit(status ? EXIT_SUCCESS : EXIT_FAILURE);
}
