
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#include <stdlib.h>
#include <assert.h>

#include "nfa.h"
#include "parse.h"
#include "xmalloc.h"

struct nfa_t *main_nfa = NULL;
unsigned long nfas_size = 0;
unsigned long nb_nfas = 0;
struct nfa_t **nfas = NULL;


void gen_nfa(void)
{
	assert(pattern != NULL);
	assert(main_nfa == NULL);
	assert(nb_nfas == 0);
	assert(nfas_size == 0);
	assert(nfas == NULL);

	nfa_build( pattern, &main_nfa );

	if (nb_nfas < nfas_size)
	{
		nfas_size = nb_nfas;
		nfas = xrealloc(nfas, nfas_size * sizeof(struct nfa_t *));
	}

	return;
}


void nfa_build(struct pattern_t *pat, struct nfa_t **nfa)
{
	struct nfa_t *nfap;
	struct nfa_state_t *final_state;

	assert(pat != NULL);
	assert(nfa != NULL);
	assert(*nfa == NULL);

	*nfa = nfa_new();
	nfap = *nfa;
	final_state = NULL;
	nfa_state_build( pat, nfa, &(nfap->start), &final_state );

	nfap->accept = nfa_state_new(state_accept, nfap);
	assert(final_state != NULL);
	assert(final_state->next1 == NULL);
	final_state->next1 = nfap->accept;

	if (nfap->nb_states < nfap->state_siz)
	{
		nfap->state_siz = nfap->nb_states;
		nfap->state = xrealloc(nfap->state, nfap->state_siz * sizeof(struct nfa_state_t *));
	}

	assert(nfap->action_executed.set_size == 0);
	set_new( &(nfap->action_executed), nfap->nb_states );

	return;
}


void nfa_state_build(struct pattern_t *root, struct nfa_t **nfa,
                     struct nfa_state_t **state_beg, struct nfa_state_t **state_end)
{
	struct nfa_state_t *beg1, *end1, *beg2, *end2, *sp;

	assert(root != NULL);
	assert(nfa != NULL);
	assert(*nfa != NULL);
	assert(state_beg != NULL);
	assert(state_end != NULL);
	assert(*state_beg == NULL);
	assert(*state_end == NULL);

	if (root->pattern1)
	{
		*state_beg = nfa_state_new(state_1line, *nfa);
		(*state_beg)->u.pattern = root;
		*state_end = *state_beg;
	}
	else
	{
		beg1 = NULL;  end1 = NULL;
		beg2 = NULL;  end2 = NULL;
		switch (root->top)
		{
			case top_star:
				*state_beg = nfa_state_new(state_epsilon, *nfa);
				nfa_state_build( root->u.n.child1, nfa, &beg1, &end1 );
				*state_end = nfa_state_new(state_epsilon, *nfa);

				assert(root->action == NULL);
				(*state_beg)->next1 = beg1;
				(*state_beg)->next2 = *state_end;
				assert(end1->next1 == NULL);
				end1->next1 = *state_end;
				(*state_end)->next2 = beg1;
				break;

			case top_plus:
				nfa_state_build( root->u.n.child1, nfa, &beg1, &end1 );
				*state_end = nfa_state_new(state_epsilon, *nfa);

				*state_beg = beg1;
				end1->next1 = *state_end;
				(*state_end)->next2 = *state_beg;
				(*state_end)->u.action = root->action;
				break;

			case top_maybe:
				*state_beg = nfa_state_new(state_epsilon, *nfa);
				nfa_state_build( root->u.n.child1, nfa, &beg1, &end1 );
				*state_end = nfa_state_new(state_epsilon, *nfa);

				assert(root->action == NULL);
				(*state_beg)->next1 = beg1;
				(*state_beg)->next2 = *state_end;
				assert(end1->next1 == NULL);
				end1->next1 = *state_end;
				break;

			case top_or:
				*state_beg = nfa_state_new(state_epsilon, *nfa);
				nfa_state_build( root->u.n.child1, nfa, &beg1, &end1 );
				nfa_state_build( root->u.n.child2, nfa, &beg2, &end2 );
				*state_end = nfa_state_new(state_epsilon, *nfa);

				(*state_beg)->next1 = beg1;
				(*state_beg)->next2 = beg2;
				assert(end1->next1 == NULL);
				assert(end2->next1 == NULL);
				end1->next1 = *state_end;
				end2->next1 = *state_end;
				(*state_end)->u.action = root->action;
				break;

			case top_and:
				*state_beg = nfa_state_new(state_conj_init, *nfa);
				sp = nfa_state_new(state_conj, *nfa);
				*state_end = nfa_state_new(state_epsilon, *nfa);

				nfa_build( root->u.n.child1, &(sp->u.s.nfa1) );
				nfa_build( root->u.n.child2, &(sp->u.s.nfa2) );

				(*state_beg)->u.s.nfa1 = sp->u.s.nfa1;
				(*state_beg)->u.s.nfa2 = sp->u.s.nfa2;

				(*state_beg)->next1 = sp;
				if (root->nullable) (*state_beg)->next2 = *state_end;
				sp->next1 = *state_end;
				(*state_end)->u.action = root->action;
				break;

			case top_concat:
				nfa_state_build( root->u.n.child1, nfa, &beg1, &end1 );
				nfa_state_build( root->u.n.child2, nfa, &beg2, &end2 );

				*state_beg = beg1;
				assert(end1->next1 == NULL);
				end1->next1 = beg2;
				if (root->action != NULL)
				{
					*state_end = nfa_state_new(state_epsilon, *nfa);
					assert(end2->next1 == NULL);
					end2->next1 = *state_end;
					(*state_end)->u.action = root->action;
				}
				else
				{
					*state_end = end2;
				}
				break;

			case top_not:
			case top_leaf:
				/* should never reach this point */
				assert(0);
				break;
		}
	}

	return;
}


struct nfa_t *nfa_new(void)
{
	struct nfa_t *this_nfa;

	this_nfa = xmalloc(sizeof(struct nfa_t));
	this_nfa->nb_states = 0;
	this_nfa->state_siz = 0;
	this_nfa->state = NULL;

	this_nfa->start = NULL;
	this_nfa->accept = NULL;

	set_new( &(this_nfa->action_executed), 0);

	if (nb_nfas == nfas_size)
	{
		if (nfas_size != 0)
		{
			assert(nfas != NULL);
			nfas_size += NFA_SIZE_INCR;
			nfas = xrealloc(nfas, nfas_size * sizeof(struct nfa_t *));
		}
		else
		{
			assert(nfas == NULL);
			nfas_size = NFA_SIZE_INIT;
			nfas = xmalloc(nfas_size * sizeof(struct nfa_t *));
		}
	}
	assert(nb_nfas < nfas_size);
	nfas[nb_nfas++] = this_nfa;

	return this_nfa;
}


struct nfa_state_t *nfa_state_new(enum state_type_t state_type, struct nfa_t *nfa)
{
	struct nfa_state_t *state;

	assert(nfa != NULL);

	state = xmalloc(sizeof(struct nfa_state_t));
	state->nfa = nfa;
	state->state_id = nfa->nb_states;
	state->visited = 0;
	state->state_type = state_type;

	switch (state_type)
	{
		case state_1line:
			state->u.pattern = NULL;
			break;

		case state_epsilon:
			state->u.action = NULL;
			break;

		case state_conj_init:
		case state_conj:
			state->u.s.nfa1 = NULL;
			state->u.s.nfa2 = NULL;
			break;

		case state_accept:
			/* noop */
			break;
	}

	state->next1 = NULL;
	state->next2 = NULL;

	if (nfa->nb_states == nfa->state_siz)
	{
		if (nfa->state_siz != 0)
		{
			assert(nfa->state != NULL);
			nfa->state_siz += NFA_SIZE_INCR;
			nfa->state = xrealloc(nfa->state, nfa->state_siz * sizeof(struct state_t *));
		}
		else
		{
			assert(nfa->state == NULL);
			nfa->state_siz = NFA_SIZE_INIT;
			nfa->state = xmalloc(nfa->state_siz * sizeof(struct state_t *));
		}
	}
	assert(nfa->nb_states < nfa->state_siz);

	nfa->state[nfa->nb_states++] = state;
	return state;
}


void nfa_free(struct nfa_t *nfa)
{
	unsigned long i;

	assert(nfa != NULL);
	assert(nfa->state_siz != 0);
	assert(nfa->state != NULL);

	set_free( &(nfa->action_executed) );

	for (i = 0; i < nfa->nb_states; i++)
	{
		assert(nfa->state[i] != NULL);
		free( nfa->state[i] );
	}
	free( nfa->state );

	free( nfa );
	return;
}
