
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#ifndef _NFA_H
#define _NFA_H

#include "pattern.h"
#include "statement.h"
#include "set.h"

#ifndef NFA_SIZE_INIT
#define NFA_SIZE_INIT	256
#endif
#ifndef NFA_SIZE_INCR
#define NFA_SIZE_INCR	NFA_SIZE_INIT
#endif


/* Non-Deterministic Finite Automata */

extern struct nfa_t *main_nfa;			/* root nfa */
extern unsigned long nfas_size;			/* size of the NFA array */
extern unsigned long nb_nfas;			/* nbr of NFAs defined */
extern struct nfa_t **nfas;			/* array of pointers to all NFAs */


struct nfa_t
{
	struct nfa_state_t *start;
	struct nfa_state_t *accept;

	unsigned long nb_states;		/* nbr of states in this NFA */
	unsigned long state_siz;		/* size of the state array */

	struct nfa_state_t **state;		/* array of pointers to all states, indexed by their ordinal */

	struct set_t action_executed;		/* prevents multiple executions */
};

struct nfa_state_t
{
	struct nfa_t *nfa;			/* this state belong to this NFA */
	unsigned long state_id;			/* unique ordinal of this state, used for indexing */
	unsigned long visited;			/* used for depth-first search */

	enum state_type_t
	{
		state_1line,			/* deterministic transition (fast evaluation) */
		state_epsilon,			/* non-deterministic epsilon transition */
		state_conj_init,		/* conjunction initialization */
		state_conj,			/* conjuction with 2 subordinated NFAs */
		state_accept			/* single accepting state */
	} state_type;

	union
	{
		struct pattern_t *pattern;	/* state_1line */
		struct statement_t *action;	/* state_epsilon */
		struct				/* state_conj_init || state_conj */
		{
			struct nfa_t *nfa1, *nfa2;
		} s;
	} u;

	struct nfa_state_t *next1, *next2;	/* next2 may be NULL */
};


void gen_nfa();

void nfa_build(struct pattern_t *pat, struct nfa_t **nfa);
void nfa_state_build(struct pattern_t *root, struct nfa_t **nfa,
                     struct nfa_state_t **state_beg, struct nfa_state_t **state_end);

struct nfa_t *nfa_new(void);
struct nfa_state_t *nfa_state_new(enum state_type_t state_type, struct nfa_t *nfa);

void nfa_free(struct nfa_t *nfa);

#endif
