
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#include <stdlib.h>
#include <assert.h>

#include "parse.h"
#include "lexic.h"

struct avl_t *decl = NULL;		/* declarations block */
struct statement_t *begin = NULL;	/* begin block */
struct pattern_t *pattern = NULL;	/* top-level pattern */
struct statement_t *success = NULL;	/* success block */
struct statement_t *failure = NULL;	/* failure block */

char *parse_err[] =
{
	/*  0 */	"integer expression expected",
	/*  1 */	"';' expected",
	/*  2 */	"'(' expected",
	/*  3 */	"')' expected",
	/*  4 */	"identifier expected",
	/*  5 */	"variable expected",
	/*  6 */	"undeclared identifier",
	/*  7 */	"'while' expected",
	/*  8 */	"'=' expected",
	/*  9 */	"array identifier expected",
	/* 10 */	"array type conflict",
	/* 11 */	"'[' expected",
	/* 12 */	"string variable expected",
	/* 13 */	"']' expected",
	/* 14 */	"statement expected",
	/* 15 */	"lvalue expected",
	/* 16 */	"type conflict in assignment",
	/* 17 */	"':' expected",
	/* 18 */	"type conflict in conditional expression",
	/* 19 */	"type conflict in relational expression",
	/* 20 */	"expression expected",
	/* 21 */	"variable or function identifier expected",
	/* 22 */	"',' expected",
	/* 23 */	"illegal number of arguments in function call",
	/* 24 */	"illegal type of argument in function call",
	/* 25 */	"end-of-file expected",
	/* 26 */	"'{' expected",
	/* 27 */	"'}' expected",
	/* 28 */	"identifier already declared",
	/* 29 */	"pattern expected",
	/* 30 */	"pattern macro identifier expected",
	/* 31 */	"statement macro identifier expected",
			NULL
};


void parse_template(void)
{
	assert(decl == NULL);
	assert(begin == NULL);
	assert(pattern == NULL);
	assert(failure == NULL);

	idtable_init( &decl );
	get_token();

	if (token == TOK_DECL)
	{
		get_token();
		parse_decl( &decl );
	}

	if (token == TOK_BEGIN)
	{
		get_token();
		parse_statement( &begin );
	}

	parse_pattern( &pattern );

	if (token == TOK_SUCCESS)
	{
		get_token();
		parse_statement( &success );
	}

	if (token == TOK_FAILURE)
	{
		get_token();
		parse_statement( &failure );
	}

	if (token != EOF) abend(parse_err[25]);

	return;
}


struct idtable_t *searchid(char *id)
{
	struct avl_t *idp;
	struct idtable_t *idt;

	assert(id != NULL);
	idp = avl_search( &decl, id, NULL );
	if (idp == NULL)
	{
		fprintf(stderr, "*** '%s'\n", id);
		abend(parse_err[6]);
	}

	idt = idp->u.idtable;

	assert(idt != NULL);
	return idt;
}
