
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#include <stdlib.h>
#include <assert.h>

#include "parse.h"
#include "evalue.h"
#include "lexic.h"
#include "xmalloc.h"
#include "options.h"


void parse_decl(struct avl_t **root)
{
	char *this_id;
	struct statement_t *this_stat;
	struct avl_t *this_decl;
	struct pattern_t *this_pat;
	int inserted;
	struct idtable_t *this_entry;
	enum vtype_t this_type;
	struct expression_t *this_expr;

	assert(root != NULL);

	if (token != '{') abend(parse_err[26]);
	get_token();

	while (token != '}')
	{
		switch (token)
		{
			case TOK_MACRO:
				this_entry = xmalloc(sizeof(struct idtable_t));
				this_entry->id_kind = id_macro;
				get_token();

				if (token != TOK_ID) abend(parse_err[4]);
				this_id = xstrdup(lval.string);
				get_token();

				assert(lexic_mode == lexic_pattern);
				if (token == '{')
				{
					this_pat = NULL;
					parse_statement( &this_stat );
					assert(this_stat != NULL);
				}
				else
				{
					this_stat = NULL;
					parse_pattern( &this_pat );
					assert(this_pat != NULL);
				}
				this_entry->u.s1.v_stat = this_stat;
				this_entry->u.s1.v_pat = this_pat;

				this_decl = avl_search( root, this_id, &inserted );
				if (! inserted) abend(parse_err[28]);
				free(this_id);

				assert(this_decl != NULL);
				this_decl->u.idtable = this_entry;
				break;

			case TOK_INT:
			case TOK_STR:
				this_type = (token == TOK_INT) ? t_int : t_str;
				do
				{
					this_entry = xmalloc(sizeof(struct idtable_t));
					get_token();

					if (token != TOK_ID) abend(parse_err[4]);
					this_id = xstrdup(lval.string);

					get_token();
					switch (token)
					{
						case '[':
							get_token();
							if (token != ']') abend(parse_err[13]);
							get_token();

							this_entry->id_kind = id_tab;
							this_entry->u.v_tab.vtype = this_type;
							this_entry->u.v_tab.root = NULL;
							break;

						case '=':
							get_token();
							this_expr = NULL;
							parse_assignment_expression( &this_expr );
							assert(this_expr != NULL);
							if (this_expr->vtype != this_type) abend(parse_err[16]);

							this_entry->id_kind = id_var;
							expr_eval(this_expr, &(this_entry->u.v_var));
							expr_free(this_expr);
							break;

						default:
							this_entry->id_kind = id_var;
							evalue_init( &(this_entry->u.v_var), this_type, opt_v_value(this_id) );
							break;
					}

					this_decl = avl_search( root, this_id, &inserted );
					if (! inserted) abend(parse_err[28]);
					free(this_id);

					assert(this_decl != NULL);
					this_decl->u.idtable = this_entry;
				} while (token == ',');
				break;

			default:
				abend(parse_err[27]);
				break;
		}

		if (token != ';') abend(parse_err[1]);
		get_token();
	}

	get_token();
	return;
}
