
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#include <stdlib.h>
#include <assert.h>

#include "parse.h"
#include "expression.h"
#include "lexic.h"
#include "xmalloc.h"


void parse_fct(struct expression_t **root)
{
	struct idtable_t *idt;
	struct expression_t *opv[FCT_MAXARGS];
	unsigned int opc, i;

	assert(root != NULL);
	assert(*root != NULL);
	assert((*root)->expr_node == n_id);
	assert((*root)->opc == 0);
	idt = (*root)->u.id;
	assert(idt != NULL);
	assert(idt->id_kind == id_fct);

	opc = 0;
	if (token == '(')
	{
		get_token();
		if (token != ')')
		{
			parse_assignment_expression( &(opv[opc++]) );
			assert(opc == 1);

			while (token != ')')
			{
				if (token != ',') abend(parse_err[22]);
				get_token();

				if (opc == FCT_MAXARGS) abend(parse_err[23]);
				parse_assignment_expression( &(opv[opc++]) );
			}

			expr_init( *root, opc );
			for (i = 0; i < opc; i++)
			{
				assert(opv[i] != NULL);
				(*root)->opv[i] = opv[i];
			}
		}
		get_token();
	}

	switch (idt->u.s2.v_fct)
	{
		case f_fflush:
			if (opc != 1) abend(parse_err[23]);
			if (opv[0]->vtype != t_int) abend(parse_err[24]);
			break;

		case f_field:	/* field( n [, LINE [, FS ]] ) */
			switch (opc)
			{
				case 3:
					if (opv[2]->vtype != t_str) abend(parse_err[24]);
					/* fall through */

				case 2:
					if (opv[1]->vtype != t_str) abend(parse_err[24]);
					/* fall through */

				case 1:
					if (opv[0]->vtype != t_int) abend(parse_err[24]);
					break;

				default:
					abend(parse_err[23]);
					break;
			}
			break;

		case f_nf:	/* NF || NF(LINE) || NF(LINE, FS) */
			switch (opc)
			{
				case 2:
					if (opv[1]->vtype != t_str) abend(parse_err[24]);
					/* fall through */

				case 1:
					if (opv[0]->vtype != t_str) abend(parse_err[24]);
					/* fall through */

				case 0:
					break;

				default:
					abend(parse_err[23]);
					break;
			}
			break;

		case f_match:	/* match(text, pattern) || match(text, pattern, flags) */
			switch (opc)
			{
				case 3:
					if (opv[2]->vtype != t_str) abend(parse_err[24]);
					/* fall through */

				case 2:
					if (opv[1]->vtype != t_str) abend(parse_err[24]);
					if (opv[0]->vtype != t_str) abend(parse_err[24]);
					break;

				default:
					abend(parse_err[23]);
					break;
			}
			break;

		case f_filename:
		case f_line:
		case f_lineno:
		case f_stderr:
		case f_stdout:
			if (opc != 0) abend(parse_err[23]);
			break;

		case f_fprintf:
			if (opc < 2) abend(parse_err[23]);
			if (opv[0]->vtype != t_int) abend(parse_err[24]);
			if (opv[1]->vtype != t_str) abend(parse_err[24]);
			/* rotate args so the file stream goes at the end */
			for (i = 1; i < opc; i++)
			{
				(*root)->opv[i - 1] = opv[i];
			}
			(*root)->opv[opc - 1] = opv[0];
			break;

		case f_index:
			if (opc != 2) abend(parse_err[23]);
			if (opv[0]->vtype != t_str) abend(parse_err[24]);
			if (opv[1]->vtype != t_str) abend(parse_err[24]);
			break;

		case f_inttostr:
			if (opc != 1) abend(parse_err[23]);
			if (opv[0]->vtype != t_int) abend(parse_err[24]);
			break;

		case f_length:
		case f_strtoint:
		case f_strtonum:
		case f_system:
		case f_tolower:
		case f_toupper:
			if (opc != 1) abend(parse_err[23]);
			if (opv[0]->vtype != t_str) abend(parse_err[24]);
			break;

		case f_printf:
		case f_sprintf:
			if (opc == 0) abend(parse_err[23]);
			if (opv[0]->vtype != t_str) abend(parse_err[24]);
			break;

		case f_split:
			switch (opc)
			{
				case 3:
					if (opv[2]->vtype != t_str) abend(parse_err[24]);
					/* fall through */

				case 2:
					if (opv[0]->vtype != t_str) abend(parse_err[24]);
					if (opv[1]->vtype != t_str) abend(parse_err[24]);
					if (opv[1]->expr_node != n_id) abend(parse_err[24]);
					idt = opv[1]->u.id;
					assert(idt != NULL);
					if (idt->id_kind != id_tab) abend(parse_err[24]);
					assert(idt->u.v_tab.vtype == opv[1]->vtype);
					break;

				default:
					abend(parse_err[23]);
					break;
			}
			break;

		case f_strcat:
			if (opc < 2) abend(parse_err[23]);
			for (i = 0; i < opc; i++)
			{
				if (opv[i]->vtype != t_str) abend(parse_err[24]);
			}
			break;

		case f_substr:
			switch (opc)
			{
				case 3:
					if (opv[2]->vtype != t_int) abend(parse_err[24]);
					/* fall through */

				case 2:
					if (opv[0]->vtype != t_str) abend(parse_err[24]);
					if (opv[1]->vtype != t_int) abend(parse_err[24]);
					break;

				default:
					abend(parse_err[23]);
					break;
			}
			break;
	}

	return;
}
