
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#include <stdlib.h>
#include <assert.h>

#include "parse.h"
#include "lexic.h"
#include "xmalloc.h"


void parse_statement(struct statement_t **root)
{
	struct expression_t *expr;
	struct stat_list_t *this_stat, *last_stat;
	struct idtable_t *idt;

	assert(root != NULL);

	switch (token)
	{
		case '{':
			*root = statement_new(st_list);
			get_token();
			last_stat = NULL;
			while (token != '}')
			{
				this_stat = xmalloc(sizeof(struct stat_list_t));
				parse_statement( &(this_stat->sp) );
				this_stat->next = NULL;

				if (last_stat != NULL)
				{
					assert(last_stat->next == NULL);
					last_stat->next = this_stat;
				}
				else
				{
					assert((*root)->u.p_list == NULL);
					(*root)->u.p_list = this_stat;
				}
				last_stat = this_stat;
			}
			get_token();
			break;

		case ';':
			*root = NULL;
			get_token();
			break;

		case TOK_IF:
			*root = statement_new(st_if);
			get_token();
			if (token != '(') abend(parse_err[2]);
			get_token();
			parse_expression( &expr );
			if (expr->vtype != t_int) abend(parse_err[0]);
			(*root)->u.p_if.expr = expr;
			if (token != ')') abend(parse_err[3]);
			get_token();
			parse_statement( &((*root)->u.p_if.p_then) );
			if (token == TOK_ELSE)
			{
				get_token();
				parse_statement( &((*root)->u.p_if.p_else) );
			}
			break;

		case TOK_WHILE:
			*root = statement_new(st_while);
			get_token();
			if (token != '(') abend(parse_err[2]);
			get_token();
			parse_expression( &expr );
			if (expr->vtype != t_int) abend(parse_err[0]);
			(*root)->u.p_while.expr = expr;
			if (token != ')') abend(parse_err[3]);
			get_token();
			parse_statement( &((*root)->u.p_while.p_body) );
			break;

		case TOK_DO:
			*root = statement_new(st_do);
			get_token();
			parse_statement( &((*root)->u.p_while.p_body) );
			if (token != TOK_WHILE) abend(parse_err[7]);
			get_token();
			if (token != '(') abend(parse_err[2]);
			get_token();
			parse_expression( &expr );
			if (expr->vtype != t_int) abend(parse_err[0]);
			(*root)->u.p_while.expr = expr;
			if (token != ')') abend(parse_err[3]);
			get_token();
			if (token != ';') abend(parse_err[1]);
			get_token();
			break;

		case TOK_FOR:
			*root = statement_new(st_for);
			get_token();
			if (token != '(') abend(parse_err[2]);
			get_token();
			if (token != ';')
			{
				parse_expression( &((*root)->u.p_for.expr1) );
			}
			if (token != ';') abend(parse_err[1]);
			get_token();
			if (token != ';')
			{
				parse_expression( &expr );
				if (expr->vtype != t_int) abend(parse_err[0]);
				(*root)->u.p_for.expr2 = expr;
			}
			if (token != ';') abend(parse_err[1]);
			get_token();
			if (token != ')')
			{
				parse_expression( &((*root)->u.p_for.expr3) );
			}
			if (token != ')') abend(parse_err[3]);
			get_token();
			parse_statement( &((*root)->u.p_for.p_body) );
			break;

		case TOK_FORALL:
			*root = statement_new(st_forall);
			get_token();
			if (token != '(') abend(parse_err[2]);
			get_token();

			if (token != TOK_ID) abend(parse_err[4]);
			idt = searchid(lval.string);
			if (idt->id_kind != id_var) abend(parse_err[5]);
			(*root)->u.p_forall.p_value = idt;
			get_token();

			if (token != '=') abend(parse_err[8]);
			get_token();

			if (token != TOK_ID) abend(parse_err[4]);
			idt = searchid(lval.string);
			if (idt->id_kind != id_tab) abend(parse_err[9]);
			if (idt->u.v_tab.vtype != (*root)->u.p_forall.p_value->u.v_var.vtype) abend(parse_err[10]);
			(*root)->u.p_forall.p_array = idt;
			get_token();

			if (token != '[') abend(parse_err[11]);
			get_token();

			if (token != TOK_ID) abend(parse_err[4]);
			idt = searchid(lval.string);
			if (idt->id_kind != id_var) abend(parse_err[5]);
			if (idt->u.v_var.vtype != t_str) abend(parse_err[12]);
			(*root)->u.p_forall.p_index = idt;
			get_token();

			if (token != ']') abend(parse_err[13]);
			get_token();

			if (token != ')') abend(parse_err[3]);
			get_token();

			parse_statement( &((*root)->u.p_forall.p_body) );
			break;

		case TOK_DELETE:
			*root = statement_new(st_delete);
			get_token();
			parse_expression( &expr );
			switch (expr->expr_node)
			{
				case n_id:
					assert(expr->u.id != NULL);
					if (expr->u.id->id_kind != id_tab) abend(parse_err[9]);
					break;
				case n_index:
					/* noop */
					break;
				default:
					abend(parse_err[9]);
			}
			(*root)->u.p_delete.expr1 = expr;
			if (token != ';') abend(parse_err[1]);
			get_token();
			break;

		case TOK_BREAK:
			*root = statement_new(st_break);
			get_token();
			if (token != ';') abend(parse_err[1]);
			get_token();
			break;

		case TOK_CONTINUE:
			*root = statement_new(st_continue);
			get_token();
			if (token != ';') abend(parse_err[1]);
			get_token();
			break;

		case TOK_ID:
			idt = searchid(lval.string);
			if (idt->id_kind == id_macro)
			{
				if (idt->u.s1.v_stat == NULL) abend(parse_err[31]);
				assert(idt->u.s1.v_pat == NULL);
				*root = idt->u.s1.v_stat;
				(*root)->ref_cnt++;

				get_token();
				if (token != ';') abend(parse_err[1]);
				get_token();
				break;
			}
			/* fall through */
			/* this ID is searched 2 times :( */

		case TOK_INTCST:
		case TOK_STRCST:
		case '(':
		case TOK_INCR:
		case TOK_DECR:
		case '+':
		case '-':
		case '~':
		case '!':
			*root = statement_new(st_expression);
			parse_expression( &((*root)->u.p_expression) );
			if (token != ';') abend(parse_err[1]);
			get_token();
			break;

		case TOK_EXIT:
			*root = statement_new(st_exit);
			get_token();
			parse_expression( &expr );
			if (expr->vtype != t_int) abend(parse_err[0]);
			(*root)->u.p_delete.expr1 = expr;
			if (token != ';') abend(parse_err[1]);
			get_token();
			break;

		case TOK_ACCEPT:
		case TOK_REJECT:
			*root = statement_new( (token == TOK_ACCEPT) ? st_accept : st_reject);
			get_token();
			if (token != ';') abend(parse_err[1]);
			get_token();
			break;

		default:
			abend(parse_err[14]);
			break;
	}

	return;
}
