
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#ifndef _PATTERN_H
#define _PATTERN_H

#include "atom.h"
#include "statement.h"


/*
   Binary tree structure representing a complete pattern.
   After normalization/optimization, the tree instance
   will be transformed into a NFA.
*/


struct pattern_t
{
	/* node type: tree operators (internal node) and operand (leaf) */
	enum top_t
	{
		/* unary operators */
		top_not,
		top_star,
		top_plus,
		top_maybe,

		/* binary operators */
		top_or,
		top_and,
		top_concat,

		/* leaf: 1-line atomic regex */
		top_leaf
	} top;

	int nullable;
	int pattern1;				/* is a 1-line pattern (fast evaluation) */

	union
	{
		struct				/* internal node */
		{
			struct pattern_t *child1, *child2;
		} n;

		struct atom_t *ap;		/* leaf */
	} u;

	struct statement_t *action;
};


struct pattern_t *pattern_new(enum top_t top);
struct pattern_t *pattern_cpy(struct pattern_t *root);

/* computes the fields "nullable" and "pattern1",
   performs also semantic checks */
void pattern_normalize(struct pattern_t *root);

/* fast evaluation of 1-line regex tree (pattern1) */
int pattern_match(char *line, size_t lg, struct pattern_t *root);

void pattern_free(struct pattern_t *root);

#endif
