
/*
hawk (Herv's awk) v1.0
Copyright (C) by Denis Valois && Cdric Llorens, 2008.
denis.valois@laposte.fr, cedric.llorens@wanadoo.fr

This software is a computer program whose purpose is to perform
pattern scanning and processing.  This is governed by a given
template containing patterns structured in regular expressions.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

#ifndef _STATEMENT_H
#define _STATEMENT_H

#include "expression.h"
#include "idtable.h"


struct stat_list_t
{
	struct statement_t *sp;
	struct stat_list_t *next;
};

struct statement_t
{
	unsigned int ref_cnt;

	enum st_type_t
	{
		st_list,
		st_expression,
		st_if,
		st_while,
		st_do,
		st_for,
		st_forall,
		st_break,
		st_continue,
		st_delete,
		st_exit,
		st_accept,
		st_reject
	} st_type;

	union
	{
		struct stat_list_t *p_list;

		struct expression_t *p_expression;

		struct
		{
			struct expression_t *expr;
			struct statement_t *p_then, *p_else;
		} p_if;

		struct
		{
			struct expression_t *expr;
			struct statement_t *p_body;
		} p_while;

		struct
		{
			struct expression_t *expr1, *expr2, *expr3;
			struct statement_t *p_body;
		} p_for;

		struct
		{
			struct idtable_t *p_array, *p_index, *p_value;
			struct statement_t *p_body;
		} p_forall;

		struct	/* also for st_exit */
		{
			struct expression_t *expr1;
		} p_delete;
	} u;
};


struct statement_t *statement_new(enum st_type_t st_type);

enum loopc_t			/* loop control */
{
	loopc_noloop,		/* processing outside of a loop */
	loopc_inloop,		/* normal processing inside a loop */
	loopc_break,		/* loop processing broken */
	loopc_continue		/* jump to next iteration */
};

void statement_interpret(struct statement_t *st, enum loopc_t *loopc);
/*  precondition: *loopc == loopc_noloop
   postcondition: *loopc == loopc_noloop */

void statement_free(struct statement_t **rootp);

#endif
